@@ -7,6 +7,23 @@ from pydantic import BaseModel, Field, PrivateAttr
 
 from pipelime.sequences import Sample, SamplesSequence
 
+if t.TYPE_CHECKING:
+    from pipelime.commands.interfaces import OutputDatasetInterface
+
+
+class PipeBuildingError(Exception):
+    @classmethod
+    def from_exc(cls, message: str, source_exc: Exception):
+        return cls(f"{message}\n--> {source_exc}")
+
+    @classmethod
+    def from_call(cls, name, sequence, args, source_exc):
+        return cls.from_exc(
+            f"Error while calling `{name}` on sequence "
+            f"`{sequence}` with arguments: {args}\n",
+            source_exc,
+        )
+
 
 def _build_op(
     src: t.Union[SamplesSequence, t.Type[SamplesSequence]],
@@ -17,20 +34,23 @@ def _build_op(
         name: str,
         args: t.Union[t.Mapping[str, t.Any], t.Sequence],
     ) -> SamplesSequence:
-        if ":" in name:
-            from pipelime.choixe.utils.imports import import_module
-
-            module_name, _, op_name = name.rpartition(":")
-            import_module(module_name)
-        else:
-            op_name = name
-
-        fn = getattr(seq, op_name)
         try:
-            return fn(**args) if isinstance(args, t.Mapping) else fn(*args)
-        except TypeError:
-            # try to call without expanding args
-            return fn(args)
+            if ":" in name:
+                from pipelime.choixe.utils.imports import import_module
+
+                module_name, _, op_name = name.rpartition(":")
+                import_module(module_name)
+            else:
+                op_name = name
+
+            fn = getattr(seq, op_name)
+            try:
+                return fn(**args) if isinstance(args, t.Mapping) else fn(*args)
+            except TypeError:
+                # try to call without expanding args
+                return fn(args)
+        except Exception as e:
+            raise PipeBuildingError.from_call(name, seq, args, e)
 
     if isinstance(ops, str):
         return _op_call(src, ops, {})
@@ -71,7 +91,7 @@ def build_pipe(
     ):
         source = _build_op(source, op_item)
     if not isinstance(source, SamplesSequence):
-        raise ValueError(
+        raise PipeBuildingError(
             f"Pipe `{repr(pipe_list)}` does not return a samples sequence instance."
         )
     return source
@@ -117,6 +137,29 @@ class DataStream(
             ),
         )
 
+    @classmethod
+    def from_output_dataset(
+        cls,
+        output_interface: "OutputDatasetInterface",
+        bidirectional: bool = False,
+    ) -> DataStream:
+        """Creates a DataStream from an input and an output interface.
+
+        Args:
+            output_interface (OutputDatasetInterface): The output interface to use.
+            bidirectional (bool, optional): Whether the stream should be readable too.
+                Defaults to False.
+
+        Returns:
+            DataStream: The created DataStream.
+        """
+        return cls(
+            input_sequence=output_interface.as_input().create_reader()
+            if bidirectional
+            else None,
+            output_pipe=output_interface.as_pipe(),
+        )
+
     @classmethod
     def read_write_underfolder(
         cls,
@@ -161,7 +204,9 @@ class DataStream(
         )
 
     @classmethod
-    def create_output_stream(cls, path: t.Union[str, Path], zfill: int = 0) -> DataStream:
+    def create_output_stream(
+        cls, path: t.Union[str, Path], zfill: int = 0
+    ) -> DataStream:
         """Creates a DataStream to write samples to a new underfolder dataset
         or update an existing one. NB: Samples cannot be read from this stream."""
         return cls(
